@@ -188,6 +188,14 @@ static uint8_t dstmac[ETH_ALEN];
 static char* payload_suffix = NULL;
 static const size_t payload_suffix_size = 4;
 
+// Must be at least big enough to capture an entire packet.
+// Longest possible:
+//   MAC: 802.1q(16) + IPv4(20) + ICMPv4(4) + timespec(18?) + 4 = 62.
+//   IP: 802.1q(16) + ARP(8) + 2xaddr_pair = 44.
+// 100 is enough.
+static const size_t pcap_snaplen = 100;
+static const size_t pcap_timeout_ms = 10;
+
 /* If there were any libnet write failures, we return error. */
 static size_t libnet_write_failures = 0;
 
@@ -241,6 +249,9 @@ int verbose = 0;  /* Increase with -v */
 /* Doesn't really need to be volatile, but doesn't hurt. */
 static volatile sig_atomic_t time_to_die = 0;
 
+void drop_landlock();
+void drop_no_new_privs();
+
 static float
 must_parse_float(const char* in, const char* what)
 {
@@ -597,6 +608,8 @@ drop_privileges(const char* drop_group)
                 gid = must_get_group(drop_group);
         }
         drop_fs_root();
+        drop_landlock();
+        drop_no_new_privs();
         drop_uid(uid, gid);
         drop_capabilities();
 #ifdef HAVE_UNVEIL
@@ -1656,13 +1669,17 @@ pingmac_recv(const char* unused, struct pcap_pkthdr *h, uint8_t *packet)
         assert(packet);
 
 	if(verbose>2) {
-		printf("arping: received response for mac ping\n");
+                printf("arping: received response for mac ping len=%d caplen=%d\n",
+                       h->len, h->caplen);
 	}
 
         getclock(&arrival);
 
         if (vlan_tag >= 0) {
-                if (h->caplen < LIBNET_ETH_H + LIBNET_IPV4_H + LIBNET_ICMPV4_H) {
+                if (h->caplen < LIBNET_802_1Q_H + LIBNET_IPV4_H
+                                + LIBNET_ICMPV4_H
+                                + sizeof(struct timespec)
+                                + payload_suffix_size) {
                         return;
                 }
                 veth = (void*)packet;
@@ -1671,7 +1688,10 @@ pingmac_recv(const char* unused, struct pcap_pkthdr *h, uint8_t *packet)
                 pkt_srcmac = veth->vlan_shost;
                 pkt_dstmac = veth->vlan_dhost;
         } else {
-                if (h->caplen < LIBNET_ETH_H + LIBNET_ARP_H + LIBNET_ICMPV4_H) {
+                if (h->caplen < LIBNET_ETH_H + LIBNET_IPV4_H
+                                + LIBNET_ICMPV4_H
+                                + sizeof(struct timespec)
+                                + payload_suffix_size) {
                         return;
                 }
                 heth = (void*)packet;
@@ -1755,10 +1775,10 @@ pingmac_recv(const char* unused, struct pcap_pkthdr *h, uint8_t *packet)
 
         const char* payload = (char*)hicmp + LIBNET_ICMPV4_ECHO_H;
         const size_t tmp = cast_ssize_size(payload - (char*)packet, NULL);
-        if (h->len < tmp) {
+        if (h->caplen < tmp) {
                 return;
         }
-        const size_t payload_size = h->len - tmp;
+        const size_t payload_size = h->caplen - tmp;
         if (payload_size < sizeof(struct timespec) + payload_suffix_size) {
                 return;
         }
@@ -2473,7 +2493,8 @@ arping_main(int argc, char **argv)
 	/*
 	 * pcap init
 	 */
-        if (!(pcap = do_pcap_open_live(ifname, 100, 10, ebuf))) {
+        if (!(pcap = do_pcap_open_live(ifname, pcap_snaplen,
+                                       pcap_timeout_ms, ebuf))) {
                 strip_newline(ebuf);
                 fprintf(stderr, "arping: pcap_open_live(): %s\n", ebuf);
 		exit(1);
